import java.math.BigInteger;

public class Rational {

    private BigInteger n;
    private BigInteger d;   // alltid d > 0

    /*
    public int gcd(int a, int b) {
        if (b == 0) { return a; }
        return gcd(b, a % b);
    }
    */

    /*
     * As can be seen in the commented code, it is basically copy-paste.
     * Instead of doing that, we should keep the main algorithm in one constructor
     * and call it from the other constructor. In this case, we let the constructor
     * taking two integers call the constructor taking two BigIntegers.
     * This also allows us to remove our written gcd method.
     */
    public Rational(int n, int d) {
        this(BigInteger.valueOf(n), BigInteger.valueOf(d));
        /*
        if (d < 0) {
            n = -n;
            d = -d;
        } else if (d == 0) {
            d = d / d; // intentionally divide by zero
        }
        int g = 1;
        if (n > 0) { g = gcd(n,d); }
        if (n < 0) { g = gcd(-n,d); }
        // Keep the old structure of the constructor, just modify where needed!
        this.n = BigInteger.valueOf(n / g);
        this.d = BigInteger.valueOf(d / g);
        */
    }

    // This constructor is basically a copy of the one taking two integers,
    // but for BigInteger instead
    public Rational(BigInteger n, BigInteger d) {
        int comparison = d.compareTo(BigInteger.ZERO);
        if (comparison == -1) {
            n = n.negate();
            d = d.negate();
        } else if (comparison == 0) {
            d = d.divide(d); // intentionally divide by zero, in reality we should throw an exception here
        }
        BigInteger g = BigInteger.ONE;
        comparison = n.compareTo(BigInteger.ZERO);
        if (comparison == 1) { g = n.gcd(d); }
        if (comparison == -1) { g = n.negate().gcd(d); }
        
        this.n = n.divide(g);
        this.d = d.divide(g);
    }

    // Translate each method to work with BigInteger instead
    public Rational add(Rational r) {
        //int n1 = this.n * r.d + r.n * this.d;
        //int d1 = this.d * r.d;
        BigInteger n1 = n.multiply(r.d).add(r.n.multiply(d));
        BigInteger d1 = d.multiply(r.d);
        return new Rational(n1,d1);
    }

    public Rational sub(Rational r) {
        //int n1 = this.n * r.d - r.n * this.d;
        //int d1 = this.d * r.d;
        BigInteger n1 = n.multiply(r.d).subtract(r.n.multiply(d));
        BigInteger d1 = d.multiply(r.d);
        return new Rational(n1,d1);
    }

    public Rational multiply(Rational r) {
        //int n1 = this.n * r.n;
        //int d1 = this.d * r.d;
        BigInteger n1 = n.multiply(r.n);
        BigInteger d1 = d.multiply(r.d);
        return new Rational(n1,d1);
    }

    public String toString() {
        return n + "/" + d;
    }

    public String floorIntegerAsString() {
        return String.valueOf(n.divide(d));
    }

    // Needed for pi approximation.
    public Rational pow(int k) {
        Rational res = new Rational(1, 1);
        for(int i = 0; i < k; i++) {
            res = res.multiply(this);
        }

        return res;
    }

}
